/*
 * Firepoint
 * A fire monitoring app
 * http://launchpad.net/firepoint
 *
 * Copyright 2011, 2012 Inessa Sofronova, Artem Kalinin, Artem Timonin, Alex Troshkov
 * <atroshko@cs.karelia.ru>
 *
 * Firepoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */

#include "network.h"

Network::Network()
{
    tmp = QDir::tempPath();
    QDir CacheDir (tmp);
    CacheDir.mkdir("frp");
    tmp = QDir::tempPath() + "/frp/";
    qDebug() << "tmp: " << tmp;
    httpRequestAborted = false;
}

/* Search file in cache and update if it is necessary */
void Network::checkCache()
{
    /* Load settings with country */
    GenSettings settings;
    settings.loadSettings();
    QString countryName = settings.getCountryName();

    /* Filename must be without spaces */
    countryName.replace(" ", "_");

    QString fileName = tmp + countryName + ".frp";

    /* Check Cache */
    if (QFile::exists(fileName))
    {
        QFileInfo info(fileName);
        QDateTime time;
        QDateTime current_time;
        time = info.created();

        /* Ask user about update if file in cache exist more than 3 hours */
        if (time.secsTo(current_time.currentDateTime()) > 10800)
            emit checkUser(); //Ask user about download file

        emit startParse(fileName);
    }
    else
        downloadFile(countryName);

    return;
}

/* GET request to server */
void Network::startRequest(QUrl url)
{
        reply = qnam.get(QNetworkRequest(url));
        connect(reply, SIGNAL(finished()),
                        this, SLOT(httpFinished()));
        connect(reply, SIGNAL(readyRead()),
                        this, SLOT(httpReadyRead()));
}

/* Create url for file downloading */
void Network::downloadFile(QString countryName)
{

    /* Obtain the required line */
    countryName.replace(" ", "_");

    //url = "http://firefly.geog.umd.edu/kml/download.php?file=" + countryName + "_24h.kml";
    url = "http://firms.modaps.eosdis.nasa.gov/active_fire/text/" + countryName + "_24h.csv";

    file = new QFile(tmp + countryName + ".frp");
    if (!file->open(QIODevice::WriteOnly))
    {
        delete file;
        qDebug() << "Can't open file to save fire points" << tmp + countryName + ".frp";
        return;
    }

    /* schedule the request */
    httpRequestAborted = false;
    startRequest(url);
}

/* Cancel file downloading */
void Network::cancelDownload()
{
    httpRequestAborted = true;
    reply->abort();
}

/* Finnish download file if all is ok and start parse */
void Network::httpFinished()
{
    if (httpRequestAborted)
    {
        if (file)
        {
            file->close();
            file->remove();
            delete file;
            file = 0;
        }
        reply->deleteLater();
        return;
    }

    file->flush();
    file->close();

    /* Server return redirect to new url */
    QVariant redirectionTarget = reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
    if (reply->error())
    {
        file->remove();
        qDebug()<< "Download failed";
        emit netError("Download failed");
        return;
    }
    else if (!redirectionTarget.isNull())
    {
        QUrl newUrl = url.resolved(redirectionTarget.toUrl());
        url = newUrl;
        reply->deleteLater();
        file->open(QIODevice::WriteOnly);
        file->resize(0);
        startRequest(url);
    }

    /* This is the end, lets start parse */
    QString fileName =  file->fileName();
    reply->deleteLater();
    reply = 0;
    delete file;
    file = 0;
    emit startParse(fileName);
}

/* this slot gets called every time the QNetworkReply has new data.
 * We read all of its new data and write it into the file.
 * That way we use less RAM than when reading it at the finished()
 * signal of the QNetworkReply*/
void Network::httpReadyRead()
{
	if (file)
		file->write(reply->readAll());

}

/* Delete file slot for signal from settings */
int Network::deleteFile()
{
    /* Load settings with country */
    GenSettings settings;
    settings.loadSettings();
    QString countryName = settings.getCountryName();

    /* Delete file if it necessary */
    file = new QFile(tmp + countryName + ".frp");
    if (!file->exists())
        return 2;
    if(file->remove())
        return 1;
    else
        return 0;
}
